"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongooseModel = void 0;
var Storage_1 = require("../Storage");
var MongooseModel = /** @class */ (function () {
    function MongooseModel(mongoose) {
        this.mongoose = mongoose;
    }
    MongooseModel.prototype.createModel = function (name, schema, options, existingModels, skipBrand) {
        var brands = [];
        if (!skipBrand) {
            brands = (Storage_1.default.getEmployeeBrands()) ? Storage_1.default.getEmployeeBrands() : [Storage_1.default.getBrand()];
            schema.add({ brand: String });
            this.addPreReadQueries(schema);
            this.addPreAggregateQueries(schema);
            this.addPreSaveQueries(schema);
            this.addPreInsertManyQueries(schema);
        }
        var modelName = (skipBrand || !Array.isArray(brands)) ? name : name + "-" + brands.join("-");
        if (!existingModels[modelName]) {
            existingModels[modelName] = this.mongoose.model(modelName, schema, options);
        }
        return existingModels[modelName];
    };
    MongooseModel.prototype.addPreCondition = function (next) {
        var brands = (Storage_1.default.getEmployeeBrands()) ? Storage_1.default.getEmployeeBrands() : [Storage_1.default.getBrand()];
        if (Array.isArray(brands)) {
            // @ts-ignore
            this.where({ brand: { $in: brands } });
        }
        next();
    };
    MongooseModel.prototype.addPreReadQueries = function (schema) {
        schema.pre('count', this.addPreCondition);
        schema.pre('find', this.addPreCondition);
        schema.pre('findOne', this.addPreCondition);
        schema.pre('update', this.addPreCondition);
        schema.pre('findOneAndUpdate', this.addPreCondition);
        schema.pre('updateOne', this.addPreCondition);
        schema.pre('updateMany', this.addPreCondition);
        schema.pre('findOneAndDelete', this.addPreCondition);
        schema.pre('findOneAndRemove', this.addPreCondition);
        schema.pre('deleteMany', this.addPreCondition);
        schema.pre('deleteOne', this.addPreCondition);
        schema.pre('remove', this.addPreCondition);
    };
    MongooseModel.prototype.addPreAggregate = function (next) {
        var brands = (Storage_1.default.getEmployeeBrands()) ? Storage_1.default.getEmployeeBrands() : [Storage_1.default.getBrand()];
        if (Array.isArray(brands)) {
            // @ts-ignore
            this.pipeline().unshift({ $match: { brand: { $in: brands } } });
        }
        next();
    };
    MongooseModel.prototype.addPreAggregateQueries = function (schema) {
        schema.pre('aggregate', this.addPreAggregate);
    };
    MongooseModel.prototype.addPreSave = function (next) {
        var brand = Storage_1.default.getBrand();
        // @ts-ignore
        if (!this.brand && brand) {
            // @ts-ignore
            this.brand = brand;
        }
        next();
    };
    MongooseModel.prototype.addPreSaveQueries = function (schema) {
        schema.pre('save', this.addPreSave);
    };
    MongooseModel.prototype.addPreInsertMany = function (next, dataArray) {
        var brand = Storage_1.default.getBrand();
        dataArray.forEach(function (data) {
            if (!data.tanentId && brand) {
                data.brand = brand;
            }
        });
        next();
    };
    MongooseModel.prototype.addPreInsertManyQueries = function (schema) {
        schema.pre('insertMany', this.addPreInsertMany);
    };
    return MongooseModel;
}());
exports.MongooseModel = MongooseModel;
